import numpy as np
import sys
import matplotlib.pyplot as plt
import math as m
import pdb
import scipy.optimize as optimize
#from scipy.interpolate import interp1d
"""
Code to analyze neutron widths from the Mazama code.
Paul Fanto
Last updated: 7/17/2017
Dependencies: numpy, scipy, matplotlib
"""
# Porter-Thomas distribution for x
def ptd(x):
	return 1./(np.sqrt(2*np.pi*x))*np.exp(-x/2.)

# Distribution of ln x, where x follows PTD
def ptd_log(y):
	x = np.exp(y)
	return x*ptd(x)

# functions for fitting

def chi2(x,nu):
	val = x**(nu/2. - 1.) * np.exp(-x/2.)
	val /= 2**(nu/2.)*m.gamma(nu/2.)
	return val

def chi2_amp(x,nu,A):
	val = A*chi2(x,nu)
	return val

def chi2_log(y,nu):
	x = np.exp(y)
	return x*chi2(x,nu)

def chi2_log_amp(y,nu,A):
	x = np.exp(y)
	return x*chi2_amp(x,nu,A)

def ptdnu(x,nu):
	val = (x/nu)**(nu/2. - 1.) * np.exp(-x/(2.*nu))
	val /= 2**(nu/2.)*m.gamma(nu/2.)
	return val

def ptdnu_log(y,nu):
	x = np.exp(y)
	return x*ptdnu(x,nu)


lines = open(sys.argv[1]).readlines()
V0 = float(lines[1].split()[0])
sp = lines[2].split()
vvals = np.array([])
for i in range(len(sp)):
	if sp[i] == '#':
		break
	vvals = np.append(vvals,float(sp[i]))

inprefix = str(lines[3].split()[0])
outpr = str(lines[4].split()[0])

print 'V0 = ',V0

D = 82.0e-6 # mean internal state level spacing
# reduced chi square values for PTD
chiptd_gnbar = np.zeros(len(vvals)); chiptd_sqrtE = np.zeros(len(vvals))
# nu and chi squared values from fits to distributions extracted with average width
nuvals_gnbar = np.zeros(len(vvals)); chisqvals_gnbar = np.zeros(len(vvals)) 
# nu and chi squared values from fits to distributions extracted with sqrt{E} assumption
nuvals_sqrtE = np.zeros(len(vvals)); chisqvals_sqrtE = np.zeros(len(vvals))

for iv in range(len(vvals)):
	v = vvals[iv]
	print 'v = ',v
	infile = inprefix  + '-v%1.1e-V%2.2f.dat' % (v,abs(V0))
	
	# read in data from 
	E0 = np.array([]); Gn = np.array([])
	with open(infile) as fl: 
		for line in fl:
			s = line.strip()
			if len(s) > 0 and s[0] != '#':
				sp = line.split()
				E0 = np.append(E0,float(sp[-2]))
				Gn = np.append(Gn,float(sp[-1]))
	
	Nr = len(E0)

	print 'Number of resonances = ',Nr
	
	
	# Find reduced widths by dividing out smooth energy background
	# step 1: histogram real energies to get reasonable energy bins

	# first, determine bins intelligently
	center = 1.0e-3; widthE = 0.5e-3
	while min(E0) < (center-0.5*widthE):
		center -= widthE
	while min(E0) > center + 0.5*widthE:
		center += widthE
	
	Evals = np.array([]); E0max = max(E0)
	while center + 0.5*widthE < E0max:
		Evals = np.append(Evals,center)
		center += widthE
	Evals = np.append(Evals,center)
	nEbins = len(Evals)
	
	# step 2: assume that f^2(E) \approx f^2(Eb[i]) in each bin.
	# Group energy-dependent widths into appropriate bins. Then, average
	# over widths in each bin.  This should yield f^2(Eb)
	Gnbar = np.zeros(nEbins)
	counts = np.zeros(nEbins)
	for i in range(Nr):
		Et = E0[i]; gnt = Gn[i]
		for j in range(nEbins):
			center = Evals[j]
			if Et >= center-0.5*widthE and Et <= center+0.5*widthE:
				Gnbar[j] += gnt
				counts[j] += 1.
				break

	if sum(counts) != Nr:
		raise Exception('Energy histogramming missed resonances')
	Gnbar /= map(float,counts) # energy background
	
	# obtain reduced widths with correct background and sqrt(E) background
	gnred_gnbar = np.zeros(Nr)
	gnred_sqrtE = np.zeros(Nr)
	for i in range(Nr):
		Et = E0[i]; gnt = Gn[i]
		for j in range(nEbins):
			center = Evals[j]
			if Et >= center-0.5*widthE and Et <= center+0.5*widthE:
				gnred_gnbar[i] = gnt/Gnbar[j]
				gnred_sqrtE[i] = gnt/np.sqrt(Evals[j])
				break
	#print 'mean red width gnbar = ', np.mean(gnred_gnbar); print 'mean red width sqrtE = ', np.mean(gnred_sqrtE)
	# normalize reduced widths by their mean
	data_gnbar = gnred_gnbar/np.mean(gnred_gnbar)
	data_sqrtE = gnred_sqrtE/np.mean(gnred_sqrtE)

	outprefix = outpr + '-v%1.1e-V%2.2f' % (v,abs(V0))
	of_redw = outprefix + 'RedWidthsFixBin.dat'
	with open(of_redw,'w') as f:
		f.write('# Reduced Widths \n')
		f.write('# Gnbar    SqrtE   \n')
		for i in range(len(data_gnbar)):
			f.write(' %12.8e '*2 % (data_gnbar[i],data_sqrtE[i]) + '\n')


	ld_gnbar = np.log(data_gnbar);
	ld_sqrtE = np.log(data_sqrtE);
	
	
	of_logredw = outprefix + 'logRedWidthsFixBin.dat'
	with open(of_logredw,'w') as f:
		f.write('# Logs of Reduced Widths \n')
		f.write('# Gnbar    SqrtE   \n')
		for i in range(len(ld_gnbar)):
			f.write(' %12.8e '*2 % (ld_gnbar[i],ld_sqrtE[i]) + '\n')

	data_gnbar.sort()
	data_sqrtE.sort()
	

	xloc = data_gnbar[0]; dx = 0.25
	'''
	while xloc - 0.5*dx > data_gnbar[0]:
		xloc -= dx
	while xloc + 0.5*dx < data_gnbar[0]:
		xloc += 0.5*dx
	'''
	xvals_gnbar = np.array([])

	while xloc + 0.5*dx < data_gnbar[-1]:
		xvals_gnbar = np.append(xvals_gnbar,xloc)
		xloc += dx
	xvals_gnbar = np.append(xvals_gnbar,xloc)
	
	nxbins_gnbar = len(xvals_gnbar)
	nx_gnbar = np.zeros(nxbins_gnbar)
	index = 0
	for i in range(nxbins_gnbar):
		xloc = xvals_gnbar[i]
		while data_gnbar[index] <= xloc + 0.5*dx:
			nx_gnbar[i] += 1.
			index += 1
			if index == Nr:
				break
	if sum(nx_gnbar) != Nr:
		raise Exception('Reduced widths gnbar missing in histogram')
	
	px_gnbar = nx_gnbar*1.0/(dx*float(Nr))

	xloc = data_sqrtE[0]
	'''
	while xloc - 0.5*dx > data_sqrtE[0]:
		xloc -= dx
	while xloc + 0.5*dx < data_sqrtE[0]:
		xloc += dx
	'''
	xvals_sqrtE = np.array([])
	while xloc + 0.5*dx < data_sqrtE[-1]:
		xvals_sqrtE = np.append(xvals_sqrtE,xloc)
		xloc += dx
	xvals_sqrtE = np.append(xvals_sqrtE,xloc)

	nxbins_sqrtE = len(xvals_sqrtE)
	nx_sqrtE = np.zeros(nxbins_sqrtE)
	index = 0
	for i in range(nxbins_sqrtE):
		xloc = xvals_sqrtE[i]
		while data_sqrtE[index] <= xloc + 0.5*dx:
			nx_sqrtE[i] += 1.
			index += 1
			if index == Nr:
				break
	if sum(nx_sqrtE) != Nr:
		raise Exception('Reduced widths sqrtE missing in histogram')

	px_sqrtE = nx_sqrtE*1.0/(dx*float(Nr))

	ld_gnbar.sort()
	ld_sqrtE.sort()

	# histogram reduced width estimates. 
	# gnbar
	logx = -10.; widthD = 0.25
	while min(ld_gnbar) < logx-0.5*widthD:
		logx -= widthD
	while min(ld_gnbar) > logx + 0.5*widthD:
		logx += widthD

	logGn_gnbar = np.array([])
	while logx + 0.5*widthD < ld_gnbar[-1]:
		logGn_gnbar = np.append(logGn_gnbar,logx)
		logx += widthD
	logGn_gnbar = np.append(logGn_gnbar,logx)
	nbins_gnbar = len(logGn_gnbar)


	counts_gnbar = np.zeros(nbins_gnbar)
	index = 0
	for i in range(nbins_gnbar):
		logx = logGn_gnbar[i]
		while ld_gnbar[index] <= logx + 0.5*widthD:
			counts_gnbar[i] += 1.
			index += 1
			if index == Nr:
				break
	#edges_gnbar[nbins_gnbar] = logx - 0.5*widthD
	#counts2_gnbar,edges_gnbar = np.histogram(ld_gnbar,bins = edges_gnbar)
	if sum(counts_gnbar) != Nr:
		raise Exception('Histogramming log of reduced widths for gnbar missed data.')

	prob_gnbar = counts_gnbar*1.0/(widthD*float(Nr))
	
	#sqrtE
	logx = -10.;
	while min(ld_sqrtE) < logx - 0.5*widthD:
		logx -= widthD
	while min(ld_sqrtE) > logx + 0.5*widthD:
		logx += widthD

	logGn_sqrtE = np.array([])
	while logx + 0.5*widthD < ld_sqrtE[-1]:
		logGn_sqrtE = np.append(logGn_sqrtE,logx)
		logx += widthD
	logGn_sqrtE = np.append(logGn_sqrtE,logx)

	nbins_sqrtE = len(logGn_sqrtE)
	counts_sqrtE = np.zeros(nbins_sqrtE)
	index = 0
	for i in range(nbins_sqrtE):
		logx = logGn_sqrtE[i]
		while ld_sqrtE[index] <= logx + 0.5*widthD:
			counts_sqrtE[i] += 1.
			index += 1
			if index == Nr:
				break


	if sum(counts_sqrtE) != Nr:
		raise Exception('Histogramming log reduced widths for sqrtE missed data.')

	prob_sqrtE = counts_sqrtE*1.0/(widthD*float(Nr))
	

	of_gnbar = outprefix + 'GnbarFixBin.dat'
	with open(of_gnbar,'w') as f:
		f.write('# Gnbar \n')
		f.write('# Number of resonances = ' + str(Nr) + '\n')
		f.write('# Evals (keV)      Gnbar (eV) \n')
		for i in range(len(Gnbar)):
			f.write((' %12.8e '*2 + '\n') %(Evals[i]*1e3,Gnbar[i]*1e6))

	of_redw = outprefix + 'RedWidthsFixBin.dat'
	with open(of_redw,'w') as f:
		f.write('# Reduced Widths \n')
		f.write('# Gnbar    SqrtE   \n')
		for i in range(len(data_gnbar)):
			f.write(' %12.8e '*2 % (data_gnbar[i],data_sqrtE[i]) + '\n')

	of_px_gnbar = outprefix + 'PxFixBin_Gnbar.dat'
	with open(of_px_gnbar,'w') as f:
		f.write('# Histogram of reduced widths, reduction A \n')
		f.write('# V0, v = %2.2f %2.2e \n' % (abs(V0),v))
		f.write('# Number of resonances = ' + str(Nr) + '\n')
		f.write('# g_n/<g_n>_Gnbar   Prob  PTD   Counts    Widths\n')
		for i in range(nxbins_gnbar):
			f.write((' %12.8e '*5 + '\n') %(xvals_gnbar[i],px_gnbar[i],ptd(xvals_gnbar[i]),nx_gnbar[i],dx) )

	of_px_sqrtE = outprefix + 'PxFixBin_sqrtE.dat'
	with open(of_px_sqrtE,'w') as f:
		f.write('# Histogram of reduced widths, reduction B \n')
		f.write('# V0, v = %2.2f %2.2e \n' % (abs(V0),v))
		f.write('# Number of resonances = ' + str(Nr) + '\n')
		f.write('# g_n/<g_n>_sqrtE   Prob  PTD   Counts    Widths\n')
		for i in range(nxbins_sqrtE):
			f.write((' %12.8e '*5 + '\n') %(xvals_sqrtE[i],px_sqrtE[i],ptd(xvals_sqrtE[i]),nx_sqrtE[i],dx) )

	of_prob_gnbar = outprefix + 'logProbsFixBin_Gnbar.dat'
	with open(of_prob_gnbar,'w') as f:
		f.write('# Histogram of logarithms of reduced widths, reduction A \n')
		f.write('# V0, v = %2.2f %2.2e \n' % (abs(V0),v))
		f.write('# Number of resonances = ' + str(Nr) + '\n')
		f.write('# log(g_n/<g_n>)_Gnbar   Prob  PTD   Counts    Widths\n')
		for i in range(nbins_gnbar):
			f.write((' %12.8e '*5 + '\n') %(logGn_gnbar[i],prob_gnbar[i],ptd_log(logGn_gnbar[i]),counts_gnbar[i],widthD))

	of_prob_sqrtE = outprefix + 'logProbsFixBin_sqrtE.dat'
	with open(of_prob_sqrtE,'w') as f:
		f.write('# Histogram of logarithms of reduced widths, reduction B \n')
		f.write('# V0, v = %2.2f %2.2e \n' % (abs(V0),v))
		f.write('# Number of resonances = ' + str(Nr) + '\n')
		f.write('# log(g_n/<g_n>)_sqrtE  Prob PTD  Counts  Widths \n')
		for i in range(nbins_sqrtE):
			f.write((' %12.8e '*5 + '\n') %(logGn_sqrtE[i],prob_sqrtE[i],ptd_log(logGn_sqrtE[i]),counts_sqrtE[i],widthD))






