import numpy as np
import sys,pdb
import math as m

"""
Maximum-likelihood fitting.
Author: Paul Fanto
"""

# chi-square function in nu degrees of freedom
def ptdnu(x,nu):
	val = nu*(nu*x/2.)**(nu/2.-1)/(2*m.gamma(nu/2.))
	val *= np.exp(-nu*x/2.)
	return val

# corresponding distribution for y = log(x) if x is distributed according to a chi-square function
# in nu degrees of freedom
def ptdnu_log(y,nu):
	x = np.exp(y)
	return x*ptdnu(x,nu)

# reduced chi-square parameter to estimate goodness of fit
def chi2(Ok,Ek,ndof):
	Nk = len(Ok)
	#print Nk
	val = 0.0
	for i in range(Nk):
		val += (Ok[i]-Ek[i])**2/Ek[i]
	val /= float(ndof)
	return val

# do maximum likelihood fits
lines = open(sys.argv[1]).readlines()
V0 = float(lines[1].split()[0])
sp = lines[2].split()
vvals = np.array([])
for i in range(len(sp)):
	if sp[i] == '#':
		break
	vvals = np.append(vvals,float(sp[i]))
prf = str(lines[4].split()[0])

Ntrials = 100; Nint = 160
numin = 0.1; numax = 2.0; dnu = 0.01
nuvals = np.array([])
nu = numin*1.0
while nu < numax + dnu:
	nuvals = np.append(nuvals,nu)
	nu += dnu

Nnu = len(nuvals)

outfile = prf + '-V%2.2f' % abs(V0) + 'LikeFit.dat'
of = open(outfile,'w') 
of.write('# V_0 = ' + str(V0) + '\n')
of.write('# v   chi2gn_ptd  chi2sq_ptd   nu_gn_fit   chi2gn_fit   nu_sq_fit   chi2sq_fit \n')
for iv in range(len(vvals)):
	v = vvals[iv]
	prefix = prf + '-v%1.1e-V%2.2f' % (v,abs(V0))
	filename = prefix + 'RedWidthsFixBin.dat'
	M = np.loadtxt(filename)
	xgn = M[:,0]; xsq = M[:,1]
	Ndata = len(xgn)

	# global analysis
	logLgn = np.zeros(Nnu); logLsq = np.zeros(Nnu)

	#chi2gnvals = np.zeros(Nnu); chi2sqvals = 
	for i in range(Nnu):
		nu = nuvals[i]
		for j in range(Ndata):
			logLgn[i] += np.log(ptdnu(xgn[j],nu))
			logLsq[i] += np.log(ptdnu(xsq[j],nu))
	nutotgn = nuvals[np.argmax(logLgn)]
	nutotsq = nuvals[np.argmax(logLsq)]

	filegn = prefix + 'logProbsFixBin_Gnbar.dat'
	Mg = np.loadtxt(filegn)
	yvalsG = Mg[:,0]; Ogn = map(float,Mg[:,3]); 
	Egn_ptd = sum(Ogn)*abs(yvalsG[1]-yvalsG[0])*ptdnu_log(yvalsG,1.0); Egn_fit = sum(Ogn)*abs(yvalsG[1]-yvalsG[0])*ptdnu_log(yvalsG,nutotgn)
	chi2gn_ptd = chi2(Ogn,Egn_ptd,len(yvalsG)-1)
	chi2gn_fit = chi2(Ogn,Egn_fit,len(yvalsG)-1)
	
	filesq = prefix + 'logProbsFixBin_sqrtE.dat'
	Ms = np.loadtxt(filesq)
	yvalsS = Ms[:,0]; Osq = map(float,Ms[:,3]); 
	Esq_ptd = sum(Osq)*abs(yvalsS[1]-yvalsS[0])*ptdnu_log(yvalsS,1.0); Esq = sum(Osq)*abs(yvalsS[1]-yvalsS[0])*ptdnu_log(yvalsS,nutotsq)
	chi2sq_ptd = chi2(Osq,Esq_ptd,len(yvalsS)-1)
	chi2sq_fit = chi2(Osq,Esq,len(yvalsS)-1)

	of.write((' %4.2e ' + ' %6.5f '*6 + '\n') %(v,chi2gn_ptd,chi2sq_ptd,nutotgn,chi2gn_fit,nutotsq,chi2sq_fit))
of.close()

	






		




