# Author: G.F. Bertsch.  Some edits made by P. Fanto
# Mazama code for S matrix
import sys,os,pdb
import numpy as np
import numpy.linalg as la
import numpy.random as random
import cmath
import math as m


# make levels between E1 and E2 follow the level distribution of the middle third 
# of a GOE spectrum
def wigner(E1,E2,N):
  if N > 240:
    print 'N too large'
    sys.exit()
  m1=np.random.randn(3*N,3*N)
  m3=m1+np.transpose(m1)
  e,u = la.eigh(m3)
  e.sort()
  # select middle third
  emax = e[2*N]
  emin = e[N]
  erange = emax-emin
  scale = (E2-E1)/erange
  eout = np.array([0.0]*N)
  for i in range(N):
     eout[i] = (e[i+N]-emin)*scale + E1
  return eout

# Make the incoming neutron Hamiltonian
# V0, a, r0 define the Woods-Saxon potential
# delr = mesh spacing
# Nmesh = number of mesh points
def mk_Hn(V0,a,r0,A,delr,Nmesh):
  R = r0*A**(1./3.)
  h2over2Mdx2 = 20.75/delr**2
  H = np.zeros((Nmesh,Nmesh))
  V = np.zeros(Nmesh)
  for i in range(Nmesh):
    rn = (i+1)*delr
    V[i] = V0/(1.0 + np.exp((rn-R)/a))
    H[i,i] = 2*h2over2Mdx2 + V[i]
    if i > 0:
        H[i,i-1] = H[i-1,i] = -h2over2Mdx2
  return H,H[i-1,i],h2over2Mdx2

# Make the diagonal CN states.  Spacings follow the middle third of a random-matrix spectrum.
def mk_Hc(Nc,E0,dE,gamb):
  Hb = np.array([[0.0j]*Nc]*Nc)
  eb = np.zeros(Nc)
  E1 = E0 + dE*Nc
  eb = wigner(E0,E1,Nc)
  for i in range(Nc):
    Hb[i,i] = eb[i] - 1.0j*gamb/2.
  return Hb

# Construct off-diagonal block
def mk_vnI(i_entry,Nn,NI,sigma):
  # Couplings exist only in entrance state i_entry
   if NI == 1:
     vnI = np.array([[0.0]*NI]*Nn)
     vnI[i_entry,0] = sigma
   else:
     tmp = sigma*random.randn(NI)
     vnI = np.array([[0.0]*NI]*Nn)
     vnI[i_entry,:] = tmp
   return vnI

def flux(u1,u2,t12):
  f = (t12/1.0j)*(np.conjugate(u1)*u2 - np.conjugate(u2)*u1)
  return f


lines = open(sys.argv[1]).readlines()

ss = lines[0].split() # array with inputs for Hamiltonian
lines = lines[1:]

# read in parameters of Hamiltonian
V0, a_r, r0, A, delr, Nn, i_entry = map(float,ss[:7])
print 'V0, a_r, r0, A = ', V0,a_r,r0,A
print 'delr, number of spatial mesh sites, interaction point = ',delr,Nn, i_entry

#
i_entry = int(i_entry) # interaction point between entrance channel and internal states
Nn = int(Nn)

# make neutron Hamiltonian
Hn,tr,h2over2Mdx2 = mk_Hn(V0,a_r,r0,A,delr,Nn)        # make Hn

seed = int(lines[0].split()[0])
print '#', seed, ' seed'
# seed random number generator
seed = int(seed)
random.seed(seed)

# read in internal state parameters
ss = lines[1].split()
Nc = int(ss[0]); EI, dE, Gg, vb = map(float,ss[1:5])

# build full Hamiltonian
print 'Nc,lowest internal energy above buffer (keV), level spacing (eV), \
 gamma width (meV), v = ',Nc,EI*1000,dE*1e6,Gg*1e9,vb

Ntot = Nn + Nc

Hc = mk_Hc(Nc,EI,dE,Gg); V = mk_vnI(i_entry,Nn,Nc,vb)

H = np.array([[0.0j]*Ntot]*Ntot)
H[0:Nn,0:Nn] = Hn
H[Nn:Ntot,Nn:Ntot] = Hc
H[0:Nn,Nn:Ntot] = V
H[Nn:Ntot,0:Nn] = np.transpose(V)

# Initialize wavefunction
vec0 = np.array([0.0]*Ntot)
vec0[Nn-1] = -tr

ss = lines[2].split()

# incoming neutron energies, in a range
E0,delE = map(float,ss[:2]); Ne = int(ss[2]); sig = ss[3]

rate = np.array([0.0]*2)
print '# neutron energies: E_low, dE, NE =', E0, delE, Ne
if sig == 'sig':
  print '#   E          (1 -|Snn|^2)   sigma_el        sigma_cap '
else:
  print '#   E      (1 -|Snn|^2)   |Sni|^2  etc'
for ie in range(Ne):
  E = ie*delE + E0
  HmE = H*1.0
  for k in range(Ntot):
    HmE[k,k] -= E

  HmEinv = la.inv(HmE)
  vec = np.dot(HmEinv,vec0)
  #for i in range(Ntot):
  #  continue
  x = vec[Nn-1]
  kdx = m.acos(1-(E/(2*h2over2Mdx2)))
  #kdx = m.acos(np.real(np.exp(1j*)))
  phase = np.e**(1.0j*kdx)
  r = -(1.-x/phase)/(1.-x*phase)
  Snn = -(1./phase)**(2*Nn)*r # s matrix element
  R = abs(r)**2
  rate[0] = R*1.0 # = |S_{nn}|^2

  u1 = 1 +r
  u2 = 1./phase + r*phase
  u1in = 1
  u2in = 1./phase
  u = vec*u2
  flux_in = flux(u1in,u2in,h2over2Mdx2).real
  #print('flux_in = ' + str(flux_in))
  W = 0.0
  for k in range(Nn,Ntot):
    W += 2*(abs(u[k])**2)*HmE[k,k].imag
  rate[1] = W/flux_in

  ss = ' %12.8e' % E
  sigma0 = 1.0
  if sig == 'sig':
    sigma0 = 3.1416/(E/20.75)/100.    # cross section in barns, = pi/k^2
  ss += ' %8.4e '*3 % (1.0-rate[0],sigma0*np.abs(1.0-Snn)**2,sigma0*rate[1])
  print ss
  #print ss
  if abs(np.sum(rate)-1.0) > 1e-6:
    print np.sum(rate)
    print 'conservation of probablity fails'
sys.exit()
